"""
EMR Calculator Main Program
"""

import sys

from emrcalc import __version__
import emrcalc.emrcalc_utilities as utl
from emrcalc.emrcalc_tk import init_tk_objects
from emrcalc.emrcalc_state import EMR_Calc_State

# Global variables
REQUIRED_MAJOR = 3
REQUIRED_MINOR = 7
MAIN_WINDOW = None
STATE_OBJECT = None

def oops(arg_string):
    """
    Log an error-string in the text window and return to caller
    """
    print("\n*** Oops, {}\n\n".format(arg_string))
    sys.exit(86)

def initialization():
    """
    Initialize the process.
    """
    ### Must be Python 3.7 or later
    if sys.version_info.major < REQUIRED_MAJOR \
    or sys.version_info.minor < REQUIRED_MINOR:
        oops("initialization: Requires Python {}.{} or later."
             .format(REQUIRED_MAJOR, REQUIRED_MINOR))

    ### Start with visible green
    frequency = 540e12 # Hz
    desc, energy, wavelen = utl.freq2info(frequency)
    state_object = EMR_Calc_State(desc, energy, frequency, wavelen)

    ### Initialize all of the Tk objects that will be needed
    window = init_tk_objects(state_object, __version__)

    ### Done, return logger handle to caller
    if state_object.FLAG_TRACING:
        print("intialization:TRACE: End")
    return state_object, window

def main():

    # ----------------------------------------------------------
    # MAIN PROGRAM
    # ----------------------------------------------------------
    ### Call process initialization.
    STATE_OBJECT, MAIN_WINDOW = initialization()

    # ----------------------------------------------------------
    ### Enter Tk mainloop
    if STATE_OBJECT.FLAG_TRACING:
        print("main:TRACE: Will now enter window mainloop")
    MAIN_WINDOW.mainloop()

    # ----------------------------------------------------------
    ### Left Tk mainloop
    if STATE_OBJECT.FLAG_TRACING:
        print("main:TRACE: Just now exited window mainloop")

if __name__ == "__main__":
    main()




"""
Initialize class emrcalc
"""

from pkg_resources import get_distribution, DistributionNotFound

try:
    __version__ = get_distribution('emrcalc').version
except DistributionNotFound:
    __version__ = '*NOT INSTALLED*'

from . import *




"""
Electromagnetic Radiation laws:
Frequency f = c / λ
Wavelength λ = c / f
Planck's Energy quanta: E = h * f = h * c / λ

where:
f = frequency in Hertz (Hz = 1/sec)
λ = wavelength in meters (m)
c = the speed of light (299,792,458 m/s)
E = energy in electron Volts (eV)
h = Plank's constant = 6.626068 * 10^-34 (m^2 * kg / s)
"""

from numpy import floor, log10
import scipy.constants as sc

# Factor for converting joules to electron volts
J2EV = 1. / sc.value(u'electron volt-joule relationship')

"""
Float-F formatter, used in float2str()
"""
FFORMATTER = lambda arg_float: "{0:f}".format(arg_float).rstrip("0").rstrip(".")

"""
Table of frequency in Hz and corresponding descriptive text
"""
EMR_TABLE = [
    [30e3, "Radio Very Low Frequency (VLF)"],
    [300e3, "Radio Low Frequency (LF)"],
    [3000e3, "Radio Medium Frequency (MF)"],
    [30e6, "Radio High Frequency (HF)"],
    [300e6, "Radio Very High Frequency (VHF)"],
    [1e9, "Radio Ultra High Frequency (UHF)"],
    [2e9, "Radio NASA Band L"],
    [4e9, "Radio NASA Band S"],
    [8e9, "Radio NASA Band C"],
    [12e9, "Radio NASA Band X"],
    [18e9, "Radio NASA Band Ku"],
    [27e9, "Radio NASA Band K"],
    [40e9, "Radio NASA Band Ka"],
    [75e9, "Radio NASA Band V"],
    [110e9, "Radio NASA Band W"],
    [300e9, "Radio Extremely High Frequency (EHF)"],
    [1.2e14, "Infrared (IR)"],
    [4.0e14, "Near Infrared (NIR)"],
    [450e12, "Visible Red"],
    [508e12, "Visible Orange"],
    [540e12, "Visible Yellow"],
    [597e12, "Visible Green"],
    [610e12, "Visible Cyan"],
    [666e12, "Visible Blue"],
    [689e12, "Visible Indigo"],
    [750e12, "Visible Violet"],
    [3e16, "Ultraviolet (UV)"],
    [1e19, "X-ray"],
    [1e21, "Gamma ray"]
    ]

def angstrom2meters(arg_angstrom):
    """
    Given a wavelength (Angstroms), return the equivalent in meters.
    """
    return arg_angstrom * 1e-10

def dump_pyobject(arg_subject, arg_object):
    """
    Dump a Python object to stdout.
    """
    for attr in dir(arg_object):
        if hasattr(arg_object, attr):
            print("%s.%s = %s" % (arg_subject, attr, getattr(arg_object, attr)))

def energy2info(arg_energy):
    """
    Given energy (J),
    return a brief description, frequency (Hz), & wavelength (meters).
    """
    freq = arg_energy / sc.h
    wavelength = sc.c / freq
    for row_freq, row_desc in EMR_TABLE:
        if freq < row_freq:
            return row_desc, freq, wavelength
    return "Cosmic ray", freq, wavelength

def float2str(arg_scalar, arg_n):
    '''
    Given a scalar and the # of desired signficant figures requested,
    convert it to a string.
    If the scalar argument cannot be converted to a float, return "-0.0".
    '''
    try:
        scalar = float(arg_scalar)
    except:
        return "0"
    if abs(scalar) == 0.0:
        return "0"
    value = round(scalar, arg_n - int(floor(log10(abs(scalar)))) - 1)
    if 1e-5 <= value <= 1e5:
        return FFORMATTER(value)
    return "{:.5G}".format(value)

def freq2info(arg_freq):
    """
    Given a frequency (Hz),
    return a brief description, energy (J), & wavelength (meters).
    """
    wavelength = sc.c / arg_freq
    joules = sc.h * arg_freq
    for row_freq, row_desc in EMR_TABLE:
        if arg_freq < row_freq:
            return row_desc, joules, wavelength
    return "Gamma", joules, wavelength

def wvlen2info(arg_wvlen):
    """
    Given a wavelength (meters),
    return a brief description, energy (J), & frequency (Hz).
    """
    freq = sc.c / arg_wvlen
    joules = sc.h * freq
    for row_freq, row_desc in EMR_TABLE:
        if freq < row_freq:
            return row_desc, joules, freq
    return "Cosmic ray", joules, freq

def test_nm_to_info(arg_nm):
    """
    Given a wavelength (nm), show brief description, frequency (Hz), and energy (eV).
    """
    xdesc, xfreq, xjoules = wvlen2info(arg_nm / 1.0e9)
    print("{} nm --> {}, {:.3e} Hz, {:.3f} J".format(arg_nm, xdesc, xfreq, xjoules))

def test1():
    """
    testing 1-2-3
    """
    # ultraviolet radiation (UV, 100 to 400 nm),
    # visible radiation (light, 400 to 700 nm),
    # and infrared radiation (IR, 700 nm to 1 mm = 10^6 nm)
    test_nm_to_info(200)
    test_nm_to_info(400)
    test_nm_to_info(700)
    test_nm_to_info(2000)



"""
EMR Calculator Class Definition for EMR_Calc_State
"""

import scipy.constants as sc

class EMR_Calc_State():
    """
    EMR Calculator State Variables
    """

    FLAG_TRACING = False
    FRAME_BG = "aquamarine"

    band_desc = "RUBBISH"
    disp_value_energy = 0.0
    disp_value_freq = 0.0
    disp_value_wvlen = 0.0

    norm_value_energy = 0.0 # J
    norm_value_freq = 0.0 # Hz
    norm_value_wvlen = 0.0 # meters

    kJ_per_mol = 0.0 # kJ/mol

    list_units_energy = ("J", "kJ", "eV")
    list_units_freq = ("Hz", "MHz", "GHz")
    list_units_wvlen = ("m", "μm", "nm", "Å")

    disp_units_energy = list_units_energy[0]
    disp_units_freq = list_units_freq[0]
    disp_units_wvlen = list_units_wvlen[0]

    def __init__(self, arg_desc, arg_energy, arg_freq, arg_wvlen):
        """
        Initialize a new EMR_Calc_State object.
        """
        self.band_desc = arg_desc
        self.norm_value_energy = self.disp_value_energy = arg_energy
        self.norm_value_freq = self.disp_value_freq = arg_freq
        self.norm_value_wvlen = self.disp_value_wvlen = arg_wvlen
        self.update_kJ_per_mol() # Initialize kJ/mol

    def update_kJ_per_mol(self):
        self.kJ_per_mol = sc.Avogadro * self.norm_value_energy * 1e-3



"""
emrcalc Tk procedures
"""
import tkinter as tk
import tkinter.ttk as ttk

from emrcalc.emrcalc_utilities import float2str, freq2info, energy2info, wvlen2info, J2EV

# Global constants:
PAD_HORZ = 2
PAD_VERT = 2
SIGFIG = 5
STATUS_OK = "Everything is nominal, Captain."
STATUS_OOPS = "This makes no sense, Captain: "
STYLE_FRAME1 = "My.TFrame"
WIDTH_FLOAT = 12
WIDTH_UNITS = 3
FONT_GENERAL = ("Ariel", 12, "normal")
FONT_LABEL = ("Ariel", 12, "normal")

# Energy units
UNITS_J = "J"
UNITS_KJ = "kJ"
UNITS_EV = "eV"

# Wavelength units
UNITS_METERS = "m"
UNITS_MICRONS = "μm"
UNITS_NM = "nm"
UNITS_ANGSTROMS = "Å"

# Frequency units
UNITS_HZ = "Hz"
UNITS_MHZ = "MHz"
UNITS_GHZ = "Ghz"

# Global variables:
ENTRY_ENERGY = None
ENTRY_FREQ = None
ENTRY_WVLEN = None
TEXT_KJMOL = None
TEXT_DESC = None
TEXT_STATUS = None
SPINBOX_ENERGY = None
SPINBOX_FREQ = None
SPINBOX_WVLEN = None
STATE_OBJECT = None
WINDOW = None

def destroyer():
    """
    Operator selected to quit to the O/S.
    """
    global WINDOW

    if STATE_OBJECT.FLAG_TRACING:
        print("destroyer:TRACE: Begin")
    if not WINDOW is None:
        if STATE_OBJECT.FLAG_TRACING:
            print("destroyer:TRACE: Destroying the main window now")
        WINDOW.destroy()
    if STATE_OBJECT.FLAG_TRACING:
        print("destroyer:TRACE: End")

def refresh_all_display_values():
    """
    Update Tk display from the normalized values in the state object.
    """
    global STATE_OBJECT, ENTRY_FREQ, ENTRY_WVLEN, ENTRY_ENERGY, TEXT_KJMOL, TEXT_DESC

    if STATE_OBJECT.FLAG_TRACING:
        print("refresh_all_display_values:TRACE: Begin")

    # --- Energy
    if STATE_OBJECT.disp_units_energy == UNITS_J:
        STATE_OBJECT.disp_value_energy = STATE_OBJECT.norm_value_energy
    else:
        if STATE_OBJECT.disp_units_energy == UNITS_KJ:
            STATE_OBJECT.disp_value_energy = STATE_OBJECT.norm_value_energy / 1e3
        else: # eV
            STATE_OBJECT.disp_value_energy = STATE_OBJECT.norm_value_energy * J2EV
    ENTRY_ENERGY.delete(0, tk.END)
    ENTRY_ENERGY.insert(0, float2str(STATE_OBJECT.disp_value_energy, SIGFIG))

    # --- kJ/mol (energy density)
    STATE_OBJECT.update_kJ_per_mol()
    TEXT_KJMOL["text"] = float2str(STATE_OBJECT.kJ_per_mol, SIGFIG)

    # --- Frequency
    if STATE_OBJECT.disp_units_freq == UNITS_HZ:
        STATE_OBJECT.disp_value_freq = STATE_OBJECT.norm_value_freq
    else:
        if STATE_OBJECT.disp_units_freq == UNITS_MHZ:
            STATE_OBJECT.disp_value_freq = STATE_OBJECT.norm_value_freq / 1e6
        else: # UNITS_GHZ
            STATE_OBJECT.disp_value_freq = STATE_OBJECT.norm_value_freq / 1e9
    ENTRY_FREQ.delete(0, tk.END)
    ENTRY_FREQ.insert(0, float2str(STATE_OBJECT.disp_value_freq, SIGFIG))

    # --- Wavelength
    if STATE_OBJECT.disp_units_wvlen == UNITS_METERS:
        STATE_OBJECT.disp_value_wvlen = STATE_OBJECT.norm_value_wvlen
    else:
        if STATE_OBJECT.disp_units_wvlen == UNITS_MICRONS:
            STATE_OBJECT.disp_value_wvlen = STATE_OBJECT.norm_value_wvlen * 1e6
        else:
            if STATE_OBJECT.disp_units_wvlen == UNITS_NM:
                STATE_OBJECT.disp_value_wvlen = STATE_OBJECT.norm_value_wvlen * 1e9
            else: # Angstroms
                STATE_OBJECT.disp_value_wvlen = STATE_OBJECT.norm_value_wvlen * 1e10
    ENTRY_WVLEN.delete(0, tk.END)
    ENTRY_WVLEN.insert(0, float2str(STATE_OBJECT.disp_value_wvlen, SIGFIG))

    # --- Band descriptor
    TEXT_DESC["text"] = str(STATE_OBJECT.band_desc)

    if STATE_OBJECT.FLAG_TRACING:
        print("refresh_all_display_values:TRACE: End")

def proc_energy_value():
    """
    The operator modified the energy value.
    """
    global STATE_OBJECT, ENTRY_ENERGY, TEXT_STATUS

    if STATE_OBJECT.FLAG_TRACING:
        print("proc_energy_value:TRACE: Begin")
    new_str_value = ENTRY_ENERGY.get()
    try:
        new_float_value = float(new_str_value)
        # new norm = old norm * ( new disp / old disp)
        STATE_OBJECT.norm_value_energy = STATE_OBJECT.norm_value_energy \
            * new_float_value / STATE_OBJECT.disp_value_energy
        # Recompute desc, frequency, & wavelength based on normalized energy
        STATE_OBJECT.band_desc, \
            STATE_OBJECT.norm_value_freq, \
            STATE_OBJECT.norm_value_wvlen = energy2info(STATE_OBJECT.norm_value_energy)
        refresh_all_display_values()
        TEXT_STATUS['text'] = STATUS_OK
    except:
        TEXT_STATUS['text'] = STATUS_OOPS + new_str_value
    if STATE_OBJECT.FLAG_TRACING:
        print("proc_energy_value:TRACE: End")

def proc_freq_value():
    """
    The operator modified the frequency value.
    """
    global STATE_OBJECT, ENTRY_FREQ, TEXT_STATUS

    if STATE_OBJECT.FLAG_TRACING:
        print("proc_freq_value:TRACE: Begin")
    new_str_value = ENTRY_FREQ.get()
    try:
        new_float_value = float(new_str_value)
        # new norm = old norm * ( new disp / old disp)
        STATE_OBJECT.norm_value_freq = STATE_OBJECT.norm_value_freq \
            * new_float_value / STATE_OBJECT.disp_value_freq
        # Recompute desc, energy, & wavelength based on normalized energy
        STATE_OBJECT.band_desc, \
            STATE_OBJECT.norm_value_energy, \
            STATE_OBJECT.norm_value_wvlen = freq2info(STATE_OBJECT.norm_value_freq)
        refresh_all_display_values()
        TEXT_STATUS['text'] = STATUS_OK
    except:
        TEXT_STATUS['text'] = STATUS_OOPS + new_str_value
    if STATE_OBJECT.FLAG_TRACING:
        print("proc_freq_value:TRACE: End")

def proc_wvlen_value():
    """
    The operator modified the wavelength value.
    """
    global STATE_OBJECT, ENTRY_WVLEN, TEXT_STATUS

    if STATE_OBJECT.FLAG_TRACING:
        print("proc_wvlen_value:TRACE: Begin")
    new_str_value = ENTRY_WVLEN.get()
    try:
        new_float_value = float(new_str_value)
        # new norm = old norm * ( new disp / old disp)
        STATE_OBJECT.norm_value_wvlen = STATE_OBJECT.norm_value_wvlen \
            * new_float_value / STATE_OBJECT.disp_value_wvlen
        # Recompute desc, energy, & wavelength based on normalized energy
        STATE_OBJECT.band_desc, \
            STATE_OBJECT.norm_value_energy, \
            STATE_OBJECT.norm_value_freq = wvlen2info(STATE_OBJECT.norm_value_wvlen)
        refresh_all_display_values()
        TEXT_STATUS['text'] = STATUS_OK
    except:
        TEXT_STATUS['text'] = STATUS_OOPS + new_str_value
    if STATE_OBJECT.FLAG_TRACING:
        print("proc_wvlen_value:TRACE: End")

def proc_energy_units():
    """
    The operator modified the energy units.
    """
    global STATE_OBJECT, ENTRY_FREQ, SPINBOX_FREQ

    new_units = SPINBOX_ENERGY.get()
    if new_units == STATE_OBJECT.disp_units_energy:
        return
    STATE_OBJECT.disp_units_energy = new_units
    refresh_all_display_values()
    ENTRY_ENERGY.focus_set()

def proc_freq_units():
    """
    The operator modified the frequency units.
    """
    global STATE_OBJECT, ENTRY_FREQ, SPINBOX_FREQ

    new_units = SPINBOX_FREQ.get()
    if new_units == STATE_OBJECT.disp_units_freq:
        return
    STATE_OBJECT.disp_units_freq = new_units
    refresh_all_display_values()
    ENTRY_FREQ.focus_set()

def proc_wvlen_units():
    """
    The operator modified the wavelength units.
    """
    global STATE_OBJECT, ENTRY_WVLEN, SPINBOX_WVLEN

    new_units = SPINBOX_WVLEN.get()
    if new_units == STATE_OBJECT.disp_units_wvlen:
        return
    STATE_OBJECT.disp_units_wvlen = new_units
    refresh_all_display_values()
    ENTRY_WVLEN.focus_set()

def init_tk_objects(arg_state_object, arg_version):
    """
    Present the operator with a set of buttons.
    """
    global WINDOW, STATE_OBJECT, \
        ENTRY_FREQ, ENTRY_WVLEN, ENTRY_ENERGY, TEXT_KJMOL, TEXT_DESC, TEXT_STATUS, \
        SPINBOX_FREQ, SPINBOX_WVLEN, SPINBOX_ENERGY

    STATE_OBJECT = arg_state_object
    if STATE_OBJECT.FLAG_TRACING:
        print("init_tk_objects:TRACE: Begin, {} {} {}"
              .format(STATE_OBJECT.disp_value_freq, \
                      STATE_OBJECT.disp_value_wvlen, \
                      STATE_OBJECT.disp_value_energy))

    # Create window.
    WINDOW = tk.Tk()
    WINDOW.attributes("-fullscreen", False)
    WINDOW.title("EMR Calculator v" + arg_version)
    WINDOW.columnconfigure(0, weight=1)

    # Change default fonts.
    WINDOW.option_add("*Font", FONT_GENERAL)
    WINDOW.option_add("*Label*Font", FONT_LABEL)

    # Set up frame.
    style = ttk.Style()
    style.configure(STYLE_FRAME1, background=STATE_OBJECT.FRAME_BG)
    frame1 = ttk.Frame(WINDOW, style=STYLE_FRAME1)
    frame1.grid(column=0, row=0, sticky=(tk.N, tk.S, tk.E, tk.W))

    # Set up heading row.
    label_heading1 = ttk.Label(frame1, text="Value")
    label_heading1.grid(column=1, row=0, padx=PAD_HORZ, pady=PAD_VERT)
    label_heading2 = ttk.Label(frame1, text="Units")
    label_heading2.grid(column=3, row=0, padx=PAD_HORZ, pady=PAD_VERT)

    # Set up frequency row.
    label_freq = ttk.Label(frame1, text="Frequency:")
    label_freq.grid(column=0, row=1, sticky=tk.E, padx=PAD_HORZ, pady=PAD_VERT)
    ENTRY_FREQ = ttk.Entry(frame1, width=WIDTH_FLOAT, justify=tk.RIGHT)
    ENTRY_FREQ.focus_set()
    ENTRY_FREQ.insert(0, float2str(STATE_OBJECT.disp_value_freq, SIGFIG))
    ENTRY_FREQ.grid(column=1, row=1, padx=PAD_HORZ, pady=PAD_VERT, sticky=(tk.E, tk.W))
    button_freq = ttk.Button(frame1, text="Update", command=proc_freq_value)
    button_freq.grid(column=2, row=1, padx=PAD_HORZ, pady=PAD_VERT)
    SPINBOX_FREQ = ttk.Spinbox(frame1, width=WIDTH_UNITS, command=proc_freq_units)
    SPINBOX_FREQ['values'] = STATE_OBJECT.list_units_freq
    SPINBOX_FREQ.set(STATE_OBJECT.disp_units_freq)
    SPINBOX_FREQ.grid(column=3, row=1, padx=PAD_HORZ, pady=PAD_VERT, sticky=(tk.E, tk.W))

    # Set up wavelength row.
    label_wvlen = ttk.Label(frame1, text="Wavelength:")
    label_wvlen.grid(column=0, row=2, sticky=tk.E, padx=PAD_HORZ, pady=PAD_VERT)
    ENTRY_WVLEN = ttk.Entry(frame1, width=WIDTH_FLOAT, justify=tk.RIGHT)
    ENTRY_WVLEN.insert(0, float2str(STATE_OBJECT.disp_value_wvlen, SIGFIG))
    ENTRY_WVLEN.grid(column=1, row=2, padx=PAD_HORZ, pady=PAD_VERT, sticky=(tk.E, tk.W))
    button_wvlen = ttk.Button(frame1, text="Update", command=proc_wvlen_value)
    button_wvlen.grid(column=2, row=2, padx=PAD_HORZ, pady=PAD_VERT)
    SPINBOX_WVLEN = ttk.Spinbox(frame1, width=WIDTH_UNITS, command=proc_wvlen_units)
    SPINBOX_WVLEN['values'] = STATE_OBJECT.list_units_wvlen
    SPINBOX_WVLEN.set(STATE_OBJECT.disp_units_wvlen)
    SPINBOX_WVLEN.grid(column=3, row=2, padx=PAD_HORZ, pady=PAD_VERT, sticky=(tk.E, tk.W))

    # Set up energy row.
    label_energy = ttk.Label(frame1, text="Energy:")
    label_energy.grid(column=0, row=3, sticky=tk.E, padx=PAD_HORZ, pady=PAD_VERT)
    ENTRY_ENERGY = ttk.Entry(frame1, width=WIDTH_FLOAT, justify=tk.RIGHT)
    ENTRY_ENERGY.insert(0, float2str(STATE_OBJECT.disp_value_energy, SIGFIG))
    ENTRY_ENERGY.grid(column=1, row=3, padx=PAD_HORZ, pady=PAD_VERT, sticky=(tk.E, tk.W))
    button_energy = ttk.Button(frame1, text="Update", command=proc_energy_value)
    button_energy.grid(column=2, row=3, padx=PAD_HORZ, pady=PAD_VERT)
    SPINBOX_ENERGY = ttk.Spinbox(frame1, width=WIDTH_UNITS, command=proc_energy_units)
    SPINBOX_ENERGY['values'] = STATE_OBJECT.list_units_energy
    SPINBOX_ENERGY.set(STATE_OBJECT.disp_units_energy)
    SPINBOX_ENERGY.grid(column=3, row=3, padx=PAD_HORZ, pady=PAD_VERT, sticky=(tk.E, tk.W))

    # Set up kJ/mol row.
    label_desc = ttk.Label(frame1, text="kJ/mol:")
    label_desc.grid(column=0, row=4, sticky=tk.E, padx=PAD_HORZ, pady=PAD_VERT)
    TEXT_KJMOL = tk.Label(frame1, justify=tk.CENTER)
    TEXT_KJMOL["text"] = float2str(STATE_OBJECT.kJ_per_mol, SIGFIG)
    TEXT_KJMOL.grid(column=1, row=4, columnspan=3, padx=PAD_HORZ, pady=PAD_VERT, sticky=(tk.E, tk.W))

    # Set up description row.
    label_desc = ttk.Label(frame1, text="EMR Band:")
    label_desc.grid(column=0, row=5, sticky=tk.E, padx=PAD_HORZ, pady=PAD_VERT)
    TEXT_DESC = tk.Label(frame1, justify=tk.CENTER)
    TEXT_DESC["text"] = str(STATE_OBJECT.band_desc)
    TEXT_DESC.grid(column=1, row=5, columnspan=3, padx=PAD_HORZ, pady=PAD_VERT, sticky=(tk.E, tk.W))

    # Set up buttons.
    button_quit = ttk.Button(frame1, text="Quit", command=destroyer)
    button_quit.grid(columnspan=3, padx=PAD_HORZ, pady=PAD_VERT)

    # Set up status line.
    TEXT_STATUS = ttk.Label(frame1, justify=tk.LEFT)
    TEXT_STATUS.grid(columnspan=4, padx=PAD_HORZ, pady=PAD_VERT, sticky=(tk.E, tk.W))
    TEXT_STATUS["text"] = STATUS_OK

    # Add row and column resizing.
    WINDOW.rowconfigure(0, weight=1)
    frame1.columnconfigure(0, weight=3)
    frame1.columnconfigure(1, weight=3)
    frame1.columnconfigure(2, weight=3)
    frame1.columnconfigure(3, weight=3)
    frame1.rowconfigure(1, weight=1)

    # All done.
    if STATE_OBJECT.FLAG_TRACING:
        print("init_tk_objects:TRACE: End")
    return WINDOW



"""
setup.py -- setup script for installing a python3-pip package.
"""
# To increment version
# Check you have ~/.pypirc filled in
# git tag x.y.z
# git push && git push --tags
# rm -rf dist; python3 setup.py sdist bdist_wheel
# auditwheel repair dist/*.whl -w dist/ (Linux)
# TEST: twine upload --repository-url https://test.pypi.org/legacy/ dist/*
# twine upload dist/*


from setuptools import setup, find_packages

__version__ = "1.3.0"

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
    install_requires = fh.readlines()

entry_points = {
    'console_scripts' :
        ['emrcalc = emrcalc.emrcalc_main:main']
}


setup(
    name="emrcalc",
    version=__version__,
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="GNU General Public License Version 3",
    install_requires=install_requires,
    entry_points=entry_points,
    packages=find_packages(),
    author="Richard Elkins",
    author_email="richard.elkins@gmail.com",
    description="Electromagnetic Radiation Calculator",
    keywords="astronomy chemistry physics",
    url="https://github.com/texadactyl/emrcalc",
    zip_safe=False,
    options={"bdist_wheel": {"universal": "1"}},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering",
    ],
)
